Attribute VB_Name = "modPrinter"
Option Compare Database
Option Explicit

Function GetDefaultPrinterQueueJobs()

'==================================================================
'Author:                 Colin Riddington
'Company:             Mendip Data Systems
'Website:               https://isladogs.co.uk
'Purpose:               Gets the number of jobs in the print queue for the default printer
'Last Updated:       19/07/2023
'Reference:             Microsoft WMI Scripting 1.2 Library
'==================================================================

'Requires the reference - Microsoft WMI Scripting 1.2 Library

Dim strComputer As String
Dim objWMIService As Object, objPrinter As Object
Dim colSettings As Variant

strComputer = "."       'local host - modify for a network printer
Set objWMIService = GetObject("winmgmts:" & "{impersonationLevel=impersonate}!\\" & strComputer & "\root\cimv2")
Set colSettings = objWMIService.ExecQuery("Select * from Win32_Printer WHERE Default = True")

For Each objPrinter In colSettings
      'print output to immediate window
      Debug.Print "Default Printer: " & objPrinter.DeviceID
      Debug.Print "Print Jobs in Queue: " & objPrinter.JobCountSinceLastReset

      GetDefaultPrinterQueueJobs = objPrinter.JobCountSinceLastReset
Next

End Function

Function GetAllPrinterQueueJobs()

'==================================================================
'Author:                 Colin Riddington
'Company:             Mendip Data Systems
'Website:               https://isladogs.co.uk
'Purpose:               Gets the number of jobs in the print queue for all installed printers
'Last Updated:       23/07/2023
'Reference:             Microsoft WMI Scripting 1.2 Library
'==================================================================

'Requires the reference - Microsoft WMI Scripting 1.2 Library

Dim strComputer As String
Dim objWMIService As Object, objPrinter As Object
Dim colSettings As Variant, i As Integer

strComputer = "."       'local host - modify for a network printer
Set objWMIService = GetObject("winmgmts:" & "{impersonationLevel=impersonate}!\\" & strComputer & "\root\cimv2")
'Set colSettings = objWMIService.ExecQuery("Select * from Win32_Printer WHERE Default = True")
Set colSettings = objWMIService.ExecQuery("Select * from Win32_Printer")

i = 0

Debug.Print "Printer Queue" & vbCrLf & "=============" & vbCrLf

For Each objPrinter In colSettings
      'print output to immediate window for all installed printers
      If objPrinter.Default = True Then
        Debug.Print "Printer: " & objPrinter.DeviceID & "  (Default)"
      Else
        Debug.Print "Printer: " & objPrinter.DeviceID
      End If
      Debug.Print "Print Jobs in Queue: " & objPrinter.JobCountSinceLastReset
      
        i = i + objPrinter.JobCountSinceLastReset

Next

GetAllPrinterQueueJobs = i
Debug.Print vbCrLf & "Total number of jobs in print queue = " & i

End Function

Function GetAllPrintersWithQueueJobs(Optional hasJobs As Boolean = False)

'==================================================================
'Author:                 Colin Riddington
'Company:             Mendip Data Systems
'Website:               https://isladogs.co.uk
'Purpose:               Gets the number of jobs in the print queue for all installed printers
'                             Optional filter for those with print jobs only
'Last Updated:       27/07/2023
'Reference:             Microsoft WMI Scripting 1.2 Library
'==================================================================

'Requires the reference - Microsoft WMI Scripting 1.2 Library

Dim strComputer As String
Dim objWMIService As Object, objPrinter As Object
Dim colSettings As Variant, i As Integer

strComputer = "."       'local host - modify for a network printer
Set objWMIService = GetObject("winmgmts:" & "{impersonationLevel=impersonate}!\\" & strComputer & "\root\cimv2")
'Set colSettings = objWMIService.ExecQuery("Select * from Win32_Printer WHERE Default = True")
Set colSettings = objWMIService.ExecQuery("Select * from Win32_Printer")

i = 0

Debug.Print "Printer Queue" & vbCrLf & "=============" & vbCrLf

For Each objPrinter In colSettings
    If objPrinter.JobCountSinceLastReset >= Abs(hasJobs) Then
          'print output to immediate window
          If objPrinter.Default = True Then
            Debug.Print "Printer: " & objPrinter.DeviceID & "  (Default)"
          Else
            Debug.Print "Printer: " & objPrinter.DeviceID
          End If
          Debug.Print "Print Jobs in Queue: " & objPrinter.JobCountSinceLastReset
    
          GetAllPrintersWithQueueJobs = objPrinter.JobCountSinceLastReset
           i = i + objPrinter.JobCountSinceLastReset
    End If
Next

GetAllPrintersWithQueueJobs = i
Debug.Print vbCrLf & "Total number of jobs in print queue = " & i

End Function


